@@ -24,60 +24,75 @@
 DCORE_BEGIN_NAMESPACE
 
 Q_DECLARE_LOGGING_CATEGORY(cfLog)
+static QString NoAppId;
 
 /*!
-    \class Dtk::Core::DConfigBackend
+@~english
+    @class Dtk::Core::DConfigBackend
     \inmodule dtkcore
 
-    \brief 配置后端的抽象接口.
+    @brief Configure the abstract interface of the backend.
 
-    所有DConfig使用的配置后端都继承此类,用户可以继承此类实现自己的配置后端.
+    All configuration backends used by DConfig inherit this class, and users can inherit this class to implement their own configuration backends.
  */
 
 /*!
-    \fn bool DConfigBackend::load(const QString &) = 0
+@~english
+    @fn bool DConfigBackend::load(const QString &) = 0
 
-    \brief 初始化后端
+    @brief Initialize the backend
 
-    \a appId 管理的配置信息key值，默认为应用程序名称
+    \a appId Managed configuration information key value, the default is the application name.
   */
 
 /*!
-    \fn bool DConfigBackend::isValid() const = 0
+@~english
+    @fn bool DConfigBackend::isValid() const = 0
 
-    \sa DConfig::isValid().
+    @sa DConfig::isValid().
 
  */
 
 /*!
-    \fn QStringList DConfigBackend::keyList() const = 0
+@~english
+    @fn QStringList DConfigBackend::keyList() const = 0
 
-    \sa DConfig::keyList()
+    @sa DConfig::keyList()
 
  */
 
 /*!
-    \fn QVariant DConfigBackend::value(const QString &key, const QVariant &fallback = QVariant()) const = 0
+@~english
+    @fn QVariant DConfigBackend::value(const QString &key, const QVariant &fallback = QVariant()) const = 0
 
-    \sa DConfig::value()
+    @sa DConfig::value()
  */
 
 /*!
-    \fn void DConfigBackend::setValue(const QString &key, const QVariant &value) = 0
+@~english
+    @fn void DConfigBackend::setValue(const QString &key, const QVariant &value) = 0
 
-    \sa DConfig::setValue()
+    @sa DConfig::setValue()
  */
 
 /*!
-    \fn void DConfigBackend::reset(const QString &key)
+@~english
+    @fn void DConfigBackend::reset(const QString &key)
 
-    \sa DConfig::reset()
+    @sa DConfig::reset()
  */
 
 /*!
-    \fn QString DConfigBackend::name() const = 0
+@~english
+    @fn QString DConfigBackend::name() const = 0
 
-    \brief 后端配置的唯一标识
+    @brief The unique identity of the backend configuration
+
+/*!
+@~english
+ @fn bool DConfigBackend::isDefaultValue(const QString &key) const = 0
+
+ @sa DConfig::isDefaultValue()
 
  */
 
@@ -85,6 +100,7 @@ DConfigBackend::~DConfigBackend()
 {
 }
 
+static QString _globalAppId;
 class Q_DECL_HIDDEN DConfigPrivate : public DObjectPrivate
 {
 public:
@@ -93,7 +109,7 @@ public:
                             const QString &name,
                             const QString &subpath)
         : DObjectPrivate(qq)
-        , appId(appId.isEmpty() ? DSGApplication::id() : appId)
+        , appId(appId)
         , name(name)
         , subpath(subpath)
     {
@@ -149,8 +165,23 @@ public:
         configCache.reset(configFile->createUserCache(getuid()));
         const QString &prefix = localPrefix();
 
-        return configFile->load(prefix) &&
-               configCache->load(prefix);
+        if (!configFile->load(prefix) || !configCache->load(prefix))
+            return false;
+
+        // generic config doesn't need to fallback to generic configration.
+        if (owner->appId == NoAppId)
+            return true;
+
+        QScopedPointer<DConfigFile> file(new DConfigFile(NoAppId, owner->name, owner->subpath));
+        const bool canFallbackToGeneric = !file->meta()->metaPath(prefix).isEmpty();
+        if (canFallbackToGeneric) {
+            QScopedPointer<DConfigCache> cache(file->createUserCache(getuid()));
+            if (file->load(prefix) && cache->load(prefix)) {
+                genericConfigFile.reset(file.take());
+                genericConfigCache.reset(cache.take());
+            }
+        }
+        return true;
     }
 
     virtual QStringList keyList() const override
@@ -160,8 +191,29 @@ public:
 
     virtual QVariant value(const QString &key, const QVariant &fallback) const override
     {
-        const QVariant &v = configFile->value(key, configCache.get());
-        return v.isValid() ? v : fallback;
+        const QVariant &vc = configFile->cacheValue(configCache.get(), key);
+        if (vc.isValid())
+            return vc;
+
+        // fallback to generic configuration, and use itself's configuration if generic isn't set.
+        if (genericConfigFile) {
+            const auto &tmp = genericConfigFile->cacheValue(genericConfigCache.get(), key);
+            if (tmp.isValid())
+                return tmp;
+        }
+        const QVariant &v = configFile->value(key);
+        if (v.isValid())
+            return v;
+        // fallback to default value of generic configuration.
+        const QVariant &vg = genericConfigFile->value(key);
+        return vg.isValid() ? vg : fallback;
+    }
+
+    virtual bool isDefaultValue(const QString &key) const override
+    {
+        // Don't fallback to generic configuration
+        const QVariant &vc = configFile->cacheValue(configCache.get(), key);
+        return !vc.isValid();
     }
 
     virtual void setValue(const QString &key, const QVariant &value) override
@@ -174,8 +226,7 @@ public:
 
     virtual void reset(const QString &key) override
     {
-        const auto &originValue = configFile->meta()->value(key);
-        setValue(key, originValue);
+        setValue(key, QVariant());
     }
 
     virtual QString name() const override
@@ -195,6 +246,8 @@ private:
 private:
     QScopedPointer<DConfigFile> configFile;
     QScopedPointer<DConfigCache> configCache;
+    QScopedPointer<DConfigFile> genericConfigFile;
+    QScopedPointer<DConfigCache> genericConfigCache;
     DConfigPrivate* owner;
     const QByteArray envLocalPrefix = qgetenv("DSG_DCONFIG_FILE_BACKEND_LOCAL_PREFIX");
 };
@@ -210,6 +263,14 @@ FileBackend::~FileBackend()
         configFile->save(prefix);
         configFile.reset();
     }
+    if (genericConfigCache) {
+        genericConfigCache->save(prefix);
+        genericConfigCache.reset();
+    }
+    if (genericConfigFile) {
+        genericConfigFile->save(prefix);
+        genericConfigFile.reset();
+    }
 }
 
 #ifndef D_DISABLE_DBUS_CONFIG
@@ -249,10 +310,11 @@ public:
     }
 
     /*!
+    @~english
       \internal
 
-      初始化DBus连接,会先调用acquireManager动态获取一个配置连接,
-      再通过这个配置连接进行配置文件的访问.
+        Initialize the DBus connection, the call acquireManager dynamically obtains a configuration connection,
+        The configuration file is then accessed through this configuration connection.
      */
     virtual bool load(const QString &/*appId*/) override
     {
@@ -331,14 +393,34 @@ public:
         return decodeQDBusArgument(reply.value().variant());
     }
 
+    virtual bool isDefaultValue(const QString &key) const override
+    {
+        auto reply = config->isDefaultValue(key);
+        reply.waitForFinished();
+        if (reply.isError()) {
+            qWarning() << "Failed to call `isDefaultValue`, key:" << key
+                       << ", error message:" << reply.error().message();
+            return false;
+        }
+        return reply.value();
+    }
+
     virtual void setValue(const QString &key, const QVariant &value) override
     {
-        config->setValue(key, QDBusVariant(value));
+        auto reply = config->setValue(key, QDBusVariant(value));
+        reply.waitForFinished();
+        if (reply.isError())
+            qCWarning(cfLog) << "Failed to setValue for the key:" << key
+                             << ", error message:" << reply.error();
     }
 
     virtual void reset(const QString &key) override
     {
-        config->reset(key);
+        auto reply = config->reset(key);
+        reply.waitForFinished();
+        if (reply.isError())
+            qCWarning(cfLog) << "Failed to reset for the key:" << key
+                             << ", error message:" << reply.error();
     }
 
     virtual QString name() const override
@@ -425,12 +507,13 @@ DConfigPrivate::~DConfigPrivate()
 }
 
 /*!
+@~english
   \internal
 
-    \brief 创建一个配置后端
+    @brief Create a configuration backend
 
-    默认使用的配置后端会优先根据环境变量来选择配置中心的D-Bus接口还是文件配置后端接口。
-    若没有配置此环境变量，则根据是否有配置中心提供D-Bus服务来选择配置中心服务还是文件配置后端接口.
+    The default configuration backend preferentially selects the D-Bus interface in the configuration center or the file configuration backend interface based on environment variables.
+    If this environment variable is not configured, the configuration center service or file configuration backend interface will be selected according to whether the configuration center provides D-Bus services
  */
 DConfigBackend *DConfigPrivate::getOrCreateBackend()
 {
@@ -462,11 +545,12 @@ DConfigBackend *DConfigPrivate::getOrCreateBackend()
 }
 
 /*!
+@~english
   \internal
 
-    \brief 创建一个配置后端
+    @brief Create a configuration backend
 
-    尝试根据环境变量来选择配置中心的D-Bus接口还是文件配置后端接口。
+    Try to choose between configuring the D-Bus interface in the center or the file configuration backend interface based on the environment variables. 
  */
 DConfigBackend *DConfigPrivate::createBackendByEnv()
 {
@@ -501,21 +585,24 @@ DConfigBackend *DConfigPrivate::createBackendByEnv()
 }
 
 /*!
-    \class Dtk::Core::DConfig
+@~english
+    @class Dtk::Core::DConfig
     \inmodule dtkcore
 
-    \brief 配置策略提供的接口类
+    @brief Configure the interface class provided by the policy
 
-    此接口规范定义了开发库所提供的关于配置文件读写的相关接口，
-    如果应用程序所使用的开发库实现了此规范，则程序应当优先使用开发库提供的接口。
+    
+    This interface specification defines the relevant interfaces provided by the development library for reading and writing configuration files,
+    If the application uses a development library that implements this specification, the application should use the interfaces provided by the development library first.
  */
 
 
 /*!
- * \brief 构造配置策略提供的对象
- * \a name 配置文件名
- * \a subpath 配置文件对应的子目录
- * \a parent 父对象
+@~english
+ * @brief Constructs the objects provided by the configuration policy
+ * \a name Configuration File Name
+ * \a subpath Subdirectory corresponding to the configuration file
+ * \a parent Parent object
  */
 DConfig::DConfig(const QString &name, const QString &subpath, QObject *parent)
     : DConfig(nullptr, name, subpath, parent)
@@ -523,36 +610,74 @@ DConfig::DConfig(const QString &name, const QString &subpath, QObject *parent)
 }
 
 DConfig::DConfig(DConfigBackend *backend, const QString &name, const QString &subpath, QObject *parent)
-    : DConfig(backend, QString(), name, subpath, parent)
+    : DConfig(backend, _globalAppId.isEmpty() ? DSGApplication::id() : _globalAppId, name, subpath, parent)
 {
 
 }
+
 /*!
- * \brief 构造配置策略提供的对象, 指定配置所属的应用Id
+@~english
+ * @brief Constructs the object provided by the configuration policy, specifying the application Id to which the configuration belongs.
  * \a appId
  * \a name
  * \a subpath
  * \a parent
- * \return 构造的配置策略对象，由调用者释放
+ * @return The constructed configuration policy object, which is released by the caller
+ * @note \a appId is not empty.
  */
 DConfig *DConfig::create(const QString &appId, const QString &name, const QString &subpath, QObject *parent)
 {
+    Q_ASSERT(appId != NoAppId);
     return new DConfig(nullptr, appId, name, subpath, parent);
 }
 
 DConfig *DConfig::create(DConfigBackend *backend, const QString &appId, const QString &name, const QString &subpath, QObject *parent)
 {
+    Q_ASSERT(appId != NoAppId);
     return new DConfig(backend, appId, name, subpath, parent);
 }
 
 /*!
- * \brief 使用自定义的配置策略后端构造对象
- * \a backend 调用者继承于DConfigBackend的配置策略后端
- * \a appId 配置文件所属的应用Id，为空时默认为本应用Id
- * \a name 配置文件名
- * \a subpath 配置文件对应的子目录
- * \a parent 父对象
- * \note 调用者只构造backend，由DConfig释放。
+ * \brief Constructs the object, and which is application.
+ * \param name
+ * \param subpath
+ * \param parent
+ * \return Dconfig object, which is released by the caller
+ * @note It's usually used for application independent, we should use DConfig::create if the configuration is a specific application.
+ */
+DConfig *DConfig::createGeneric(const QString &name, const QString &subpath, QObject *parent)
+{
+    return new DConfig(nullptr, NoAppId, name, subpath, parent);
+}
+
+DConfig *DConfig::createGeneric(DConfigBackend *backend, const QString &name, const QString &subpath, QObject *parent)
+{
+    return new DConfig(backend, NoAppId, name, subpath, parent);
+}
+
+/*!
+ * \brief Explicitly specify application Id for config.
+ * \param appId
+ * @note It's should be called before QCoreApplication constructed.
+ */
+void DConfig::setAppId(const QString &appId)
+{
+    if (!_globalAppId.isEmpty()) {
+        qCWarning(cfLog, "`setAppId`should only be called once.");
+    }
+    _globalAppId = appId;
+    qCDebug(cfLog, "Explicitly specify application Id as appId=%s for config.", qPrintable(appId));
+}
+
+/*!
+@~english
+ * @brief Use custom configuration policy backend to construct objects
+ * \a backend The caller inherits the configuration policy backend of DConfigBackend
+ * \a appId The application Id of the configuration file. If it is blank, it will be the application Id by default
+ * \a name Configuration File Name
+ * \a subpath Subdirectory corresponding to the configuration file
+ * \a parent Parent object
+ * @note The caller only constructs backend, which is released by DConfig.
  */
 DConfig::DConfig(DConfigBackend *backend, const QString &appId, const QString &name, const QString &subpath, QObject *parent)
     : QObject(parent)
@@ -560,8 +685,6 @@ DConfig::DConfig(DConfigBackend *backend, const QString &appId, const QString &n
 {
     D_D(DConfig);
 
-    Q_ASSERT(!d->appId.isEmpty());
-
     qCDebug(cfLog, "Load config of appid=%s name=%s, subpath=%s",
             qPrintable(d->appId), qPrintable(d->name), qPrintable(d->subpath));
 
@@ -575,9 +698,10 @@ DConfig::DConfig(DConfigBackend *backend, const QString &appId, const QString &n
 }
 
 /*!
- * \brief DConfig::backendName
- * \return 配置策略后端名称
- * \note 调用者只能用DConfig访问DConfigBackend对象，所以不返回DConfigBackend对象。
+@~english
+ * @brief DConfig::backendName
+ * @return Configure policy backend name
+ * @note The caller can only access the DConfigBackend object with DConfig, so the DConfigBackend object is not returned.
  */
 QString DConfig::backendName() const
 {
@@ -589,8 +713,9 @@ QString DConfig::backendName() const
 }
 
 /*!
- * \brief 获得所有可用的配置项名称
- * \return 配置项名称集合
+@~english
+ * @brief Get all available configuration item names
+ * @return Configuration item name collection
  */
 QStringList DConfig::keyList() const
 {
@@ -602,8 +727,9 @@ QStringList DConfig::keyList() const
 }
 
 /*!
- * \brief 判断此后端是否可用
- * \return
+@~english
+ * @brief Check whether the backend is available
+ * @return
  */
 bool DConfig::isValid() const
 {
@@ -612,10 +738,25 @@ bool DConfig::isValid() const
 }
 
 /*!
- * \brief 根据配置项名称获得对应值
- * \param key 配置项名称
- * \param fallback 没有获取到配置项值后提供的默认值
- * \return
+@~english
+ * @brief Check whether the value is default according to the configuration item name
+ * @param key Configuration Item Name
+ * @return Return `true` if the value isn't been set, otherwise return `false`
+ */
+bool DConfig::isDefaultValue(const QString &key) const
+{
+    D_DC(DConfig);
+    if (d->invalid())
+        return false;
+    return d->backend->isDefaultValue(key);
+}
+
+/*!
+@~english
+ * @brief Get the corresponding value according to the configuration item name
+ * @param key Configuration Item Name
+ * @param fallback The default value provided after the configuration item value is not obtained
+ * @return
  */
 QVariant DConfig::value(const QString &key, const QVariant &fallback) const
 {
@@ -627,9 +768,10 @@ QVariant DConfig::value(const QString &key, const QVariant &fallback) const
 }
 
 /*!
- * \brief 根据配置项名称设置其值
- * \param 配置项名称
- * \param 需要更新的值
+@~english
+ * @brief Set the value according to the configuration item name
+ * @param key Configuration Item Name
+ * @param value Values that need to be updated
  */
 void DConfig::setValue(const QString &key, const QVariant &value)
 {
@@ -641,8 +783,9 @@ void DConfig::setValue(const QString &key, const QVariant &value)
 }
 
 /*!
- * \brief 设置其配置项对应的默认值，此值为经过override机制覆盖后的值，不一定为此配置文件中meta中定义的值
- * \param 配置项名称
+@~english
+ * @brief Set the default value corresponding to its configuration item. This value is overridden by the override mechanism. It is not necessarily the value defined in the meta in this configuration file
+ * @param key Configuration Item Name
  */
 void DConfig::reset(const QString &key)
 {
@@ -654,8 +797,9 @@ void DConfig::reset(const QString &key)
 }
 
 /*!
- * \brief 返回配置文件名称
- * \return
+@~english
+ * @brief Return configuration file name
+ * @return
  */
 QString DConfig::name() const
 {
@@ -664,8 +808,9 @@ QString DConfig::name() const
 }
 
 /*!
- * \brief 返回配置文件对应的子目录
- * \return
+@~english
+ * @brief Return the subdirectory corresponding to the configuration file
+ * @return
  */
 QString DConfig::subpath() const
 {
